/*
	(C) 2011. Kruchinin Aleksandr. http://vidikon.com
	manager.cpp - console management and run processes of PRS-RT
	1.00
*/
#include "Management.h"
#include "GetPutData.h"

#define MAX_RECOG_PROC		20			// Max recognition process

// Mutexes
HANDLE Mutex_Management;
HANDLE Mutex_Data;

int ActiveProcess=0;
PROCESS_INFORMATION pi[MAX_RECOG_PROC];
const char* PRS_RT="PRS-RT 1.00\n";
const char* author="(C) 2011. Kruchinin Aleksandr. http://vidikon.com\n";

void CreateRecogProcess(int all);
void CloseRecogProcess(int all);
DWORD WINAPI ReRunProcess(LPVOID);

int main()
{
	DWORD lpT;
	printf(PRS_RT);
	printf(author);		

	// Create thread for Management
	Mutex_Management=CreateMutex(NULL,true,"Mutex_Management");
	HANDLE H_Management=CreateThread(NULL,0,M_Management,NULL,0,&lpT);
	printf("Create module of management in thread\n");
	
	// Create thread for generate data
	Mutex_Data=CreateMutex(NULL,true,"Mutex_Data");
	HANDLE H_Data=CreateThread(NULL,0,M_Data,NULL,0,&lpT);
	printf("Create module of get and put data in thread	\n");

	// Create modules for recognition			
	int all_pi=5;	
	char buf[256];	
	CreateRecogProcess(all_pi);
	printf("Create %d processes for recognition\n\n",all_pi);

	int dt=1000;
	int dsize=100;
	bool run=0;
	bool display=0;

	//Go in to command interpretator
	char str[21];		
	do{
		if (!display) printf("PRS-RT#");
		scanf("%20s", str);		
	
		// Exit
		if (strcmp(str,"exit")==0) break;		

	  if (!run){
		// Create and close processes for recognition
		if (strcmp(str,"recog")==0) 
		{
			if (all_pi>0) {
				CloseRecogProcess(all_pi);
				all_pi=0;
				printf("Close processes for recognition\n");
			}	
			else
			{
				printf("RECOG COUNT$");
				scanf("%20s", str);
				all_pi=atoi(str);
				if (all_pi<0 || all_pi>MAX_RECOG_PROC) all_pi=0;
				CreateRecogProcess(all_pi);
				printf("Create %d processes for recognition\n",all_pi);
			}
			continue;
		}

		// Set delta time
		if (strcmp(str,"time")==0) 
		{
			printf("DTIME$");
			scanf("%20s", str);
			dt=atoi(str);
			if (dt<1) dt=1;
			SetM_DataTime(dt/2);
			SetM_ControlTime(dt);
			continue;
		}

		// Set size data
		if (strcmp(str,"size")==0) 
		{
			printf("DSIZE$");
			scanf("%20s", str);
			dsize=atoi(str);
			if (dsize<1) dsize=1;
			SetM_DataSize(dsize);
			continue;
		}

		// Run model
		if (strcmp(str,"run")==0) 
		{
			// Unblock processes and threads
			ReleaseMutex(Mutex_Management);
			ReleaseMutex(Mutex_Data);			
			Sleep(10);
			run=1;
			continue;
		}
	  }
	  else
	  {
		  // View runing model
		  if (strcmp(str,"display")==0) 
		  {
			  display=1;
			  SetM_DataDisplay();
			  SetM_ControlDisplay();
			  continue;
		  }		  
		  if (display && strcmp(str,"q")==0)
		  {
			  display=0;
			  SetM_DataDisplay();
			  SetM_ControlDisplay();
			  continue;
		  }
	  }
		printf("Unknown command\n");
	}while(1);

	CloseMemoryData(); // Free memory of data
	CloseRecogProcess(all_pi);
	CloseHandle(H_Management);		
	Sleep(1);
	return 0;
}

void CreateRecogProcess(int all)
{
	STARTUPINFO cif;
	ZeroMemory(&cif,sizeof(STARTUPINFO));	
	int i;	
	char buf[20];
	for(i=0;i<all;i++){
		wsprintf(buf,"recog.exe %d",i+1);
		CreateProcess("recog.exe",buf,
			NULL,NULL,FALSE,NULL,NULL,NULL,&cif,&pi[i]);
	}
}

void CloseRecogProcess(int all)
{
	for(int i=0;i<all;i++)
		if (pi[i].hProcess!=NULL) TerminateProcess(pi[i].hProcess,NO_ERROR);
}

int num_rerun;

void ReRunProcess(int num)
{
	num_rerun=num;
	DWORD lpT;
	HANDLE H=CreateThread(NULL,0,ReRunProcess,NULL,0,&lpT);
	CloseHandle(H);	
}

DWORD WINAPI ReRunProcess(LPVOID)
{
	STARTUPINFO cif;
	ZeroMemory(&cif,sizeof(STARTUPINFO));	
	int i;	
	char buf[20];	
	wsprintf(buf,"recog.exe %d",num_rerun+1);
	CreateProcess("recog.exe",buf,
		NULL,NULL,FALSE,NULL,NULL,NULL,&cif,&pi[num_rerun]);	
	return 0;
}